@@ -1,6 +1,8 @@
 import curses
 from collections.abc import Iterable
+import logging
 
+logger = logging.getLogger("pytermwindows")
 
 class Window:
     """
@@ -160,6 +162,7 @@ class Window:
             while True:
             
                 self.contents(**kwargs)
+                self.auto_adjust_size()
 
         except KeyboardInterrupt:
             print( "Exiting..." )
@@ -237,7 +240,7 @@ class Window:
         """
         return self.children.get( name, None )
 
-    def write( self, line : int, pos : int, string : str, refresh : bool = False ):
+    def write( self, line : int, pos : int, string : str, refresh : bool = False, clear : bool = False ):
         """
         Write a string to the window.
 
@@ -251,10 +254,12 @@ class Window:
             The string to write.
         refresh : bool
             Whether to refresh the window after writing.
+        clear : bool
+            Whether to clear the line before writing.
         """
-        self.write_to( self.window, line, pos, string, refresh )
+        self.write_to( self.window, line, pos, string, refresh, clear )
     
-    def write_to( self, window_or_pane, line : int, pos : int, string : str, refresh : bool = False ):
+    def write_to( self, window_or_pane, line : int, pos : int, string : str, refresh : bool = False, clear : bool = False ):
         """
         Write a string to a specific (sub)window or pane.
 
@@ -270,16 +275,31 @@ class Window:
             The string to write.
         refresh : bool
             Whether to refresh the window after writing.
+        clear : bool
+            Whether to clear the line before writing.
         """
+
         if self.is_linewise_iterable( line, string ):
             for l,s in zip( line, string ):
+                if clear: 
+                    window_or_pane.move( l, 0 )
+                    window_or_pane.clrtoeol()
+
                 self._write_to( window_or_pane, l, pos, s, refresh )
                 self._modify_line( self.line_diff( l ) )
 
         elif self.is_columwise_iterable( pos, string ):
+            if clear: 
+                window_or_pane.move( line, 0 )
+                window_or_pane.clrtoeol()
+                
             for p,s in zip( pos, string ):
                 self._write_to( window_or_pane, line, p, s, refresh )
+
         else:
+            if clear:
+                window_or_pane.move( line, 0 )
+                window_or_pane.clrtoeol()
             self._write_to( window_or_pane, line, pos, string, refresh )
 
     def auto_adjust_size( self ):
@@ -300,7 +320,9 @@ class Window:
         width : int
             The new width of the window (in numbers of characters per line).
         """
-        self.window.resize( height, width )
+        curses.resizeterm( height, width )
+        # self.window.resize( height, width )
+        self.window.clear()
         self._upack_size()
         self.window.refresh()
 
@@ -324,7 +346,40 @@ class Window:
         color : str
             The color to use.
         """
-        return ( text, self.colors[ color ] )
+        if not self._use_color:
+            logger.warning( "Color is not enabled." )
+            return text
+        col = self.colors.get( color, None )
+        if col:
+            return ( text, col )
+        else:
+            raise ValueError( f"Color {color} not found. Use one of these colors: { list( self.colors.keys() ) }." )
+    
+    def clear_line( self, line : int or Iterable ):
+        """
+        Clear a line.
+
+        Note
+        ----
+        This will not refresh the window. And not affect the positioning of the current_line.
+
+        Parameters
+        ----------
+        line : int or Iterable
+            The line(s) to clear. 
+        """
+        # current = self._lines
+
+        if isinstance( line, int):
+            self.window.move( line, 0 )
+            self.window.clrtoeol()
+
+        elif isinstance( line, Iterable ):
+            for l in line:
+                self.window.move( l, 0 )
+                self.window.clrtoeol()
+                
+        # self._lines = current
 
     @staticmethod
     def key_to_int( key ) -> int:
@@ -407,7 +462,6 @@ class Window:
         """
         return self._is_ab_iterable( pos, string )
 
-    
     def update_counter( self ):
         """
         Increment the counter value.
@@ -480,11 +534,33 @@ class Window:
         self._lines = self._first_line
         return self._lines
 
+    @property
+    def bottom_line( self ) -> int:
+        """
+        Get the bottom line of the window.
+        """
+        return self.height - 1
+    
     @property
     def to_bottom_line( self ) -> int:
         """
         Go to the bottom line of the window.
         """
+        self._lines = self.bottom_line
+        return self._lines
+
+    @property
+    def last_line( self ) -> int:
+        """
+        Get the down-most line that was written to.
+        """
+        return self._max_lines
+
+    @property
+    def to_last_line( self ) -> int:
+        """
+        Go to the down-most line that was written to.
+        """
         self._lines = self._max_lines
         return self._lines
 
@@ -540,14 +616,21 @@ class Window:
         if self._use_color:
             curses.start_color()
             curses.use_default_colors()
+            curses.init_pair( 1, curses.COLOR_WHITE, -1 )
+            curses.init_pair( 2, curses.COLOR_BLACK, -1 )
+            curses.init_pair( 3, curses.COLOR_RED, -1 )
+            curses.init_pair( 4, curses.COLOR_GREEN, -1 )
+            curses.init_pair( 5, curses.COLOR_BLUE, -1 )
+            curses.init_pair( 6, curses.COLOR_YELLOW, -1 )
+            curses.init_pair( 7, curses.COLOR_CYAN, -1 )
             self.colors = {
-                    "white" : curses.init_pair( 1, curses.COLOR_WHITE, -1 ),
-                    "black" : curses.init_pair( 2, curses.COLOR_BLACK, -1 ),
-                    "red"   : curses.init_pair( 3, curses.COLOR_RED, -1 ),
-                    "green" : curses.init_pair( 4, curses.COLOR_GREEN, -1 ),
-                    "blue"  : curses.init_pair( 5, curses.COLOR_BLUE, -1 ),
-                    "yellow": curses.init_pair( 6, curses.COLOR_YELLOW, -1 ),
-                    "cyan"  : curses.init_pair( 7, curses.COLOR_CYAN, -1 ),
+                    "white" : curses.color_pair( 1 ),
+                    "black" : curses.color_pair( 2 ),
+                    "red"   : curses.color_pair( 3 ),
+                    "green" : curses.color_pair( 4 ),
+                    "blue"  : curses.color_pair( 5 ),
+                    "yellow": curses.color_pair( 6 ),
+                    "cyan"  : curses.color_pair( 7 ),
                 }
 
         curses.noecho()
